// Color constants
const char BLUE = 0b000011, RED = 0b110000, GREEN = 0b001100,
           YELLOW = 0b001111, PURPLE = 0b110011, CYAN = 0b111100,
           WHITE = 0b111111, OFF = 0;

// Sequences are defined here
// black, red, yellow, green, blue, purple, cyan, white
const char RAINBOW[] = {19, 0b000010, 0b000011,
                        0b000111, 0b001011, 0b001111,
                        0b011010, 0b100101, 0b110000,
                        0b110001, 0b110010, 0b110011,
                        0b110110, 0b111001, 0b111100,
                        0b111101, 0b111110, 0b111111,
                        0b101010, 0b010101};

const char CHASE[] = {96,
                      OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF,
                      OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF,
                      0b010000, 0b100000, RED, RED, RED, RED, RED, RED,
                      RED, RED, RED, RED, RED, RED, 0b100000, 0b010000,

                      OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF,
                      OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF,
                      0b000100, 0b001000, GREEN, GREEN, GREEN, GREEN, GREEN, GREEN,
                      GREEN, GREEN, GREEN, GREEN, GREEN, GREEN, 0b001000, 0b000100,

                      OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF,
                      OFF, OFF, OFF, OFF, OFF, OFF, OFF, OFF,
                      0b000001, 0b000010, BLUE, BLUE, BLUE, BLUE, BLUE, BLUE,
                      BLUE, BLUE, BLUE, BLUE, BLUE, BLUE, 0b000010, 0b000001};

const char ON[] = {1, WHITE};

// Locations within a sequence
int seqLoc[4][4];
const int CHASE_START[4][4] = { {0, 1, 2, 3},
                       {7, 6, 5, 4},
                       {8, 9, 10, 11},
                       {15, 14, 13, 12} };

const int RAINBOW_START[4][4] = { {0, 1, 2, 3},
                       {1, 2, 3, 4},
                       {2, 3, 4, 5},
                       {3, 4, 5, 6} };

const int PUSH_START[4][4] = { {-1, -1, -1, -1},
                       {-1, -1, -1, -1},
                       {-1, -1, -1, -1},
                       {-1, -1, -1, -1} };

// Pick our favorite sequence.
const char* SEQUENCE = RAINBOW;

// Must be handled manually.
boolean buttonsHeld[4][4];

void setup()
{
  
  // Initialize the grid.
  for (int i = 0; i < 4; ++i)
    for (int j = 0; j < 4; ++j)
      // Change this to make a different starting pattern:
      seqLoc[i][j] = RAINBOW_START[i][j];
      
  initHardware();
}


void printButtonStatus()
{
  for (int i = 0; i < 4; ++i) {
    for (int j = 0; j < 4; ++j)
      Serial.print(getButton(i, j) ? '1' : '0');
    Serial.println();
  }
}

void updateSequence()
{
  for (int i = 0; i < 4; ++i)
    for (int j = 0; j < 4; ++j) {

      if (seqLoc[i][j] >= 0) {
        seqLoc[i][j] = (seqLoc[i][j] + 1) % SEQUENCE[0];
        setColor(i, j, SEQUENCE[seqLoc[i][j] + 1]);
      } else {
        setColor(i, j, OFF);
      }      
      
      if (getButton(i, j) && !buttonsHeld[i][j]) {
        seqLoc[i][j] = (seqLoc[i][j] == -1) ? 0 : -1;
        buttonsHeld[i][j] = 1;
      } else if (!getButton(i, j))
        buttonsHeld[i][j] = 0;
    }
      
}

void loop()
{ 
  updateSequence();
  delay(50);
}
